@@ -1,5 +1,7 @@
-use std::ops::{BitAndAssign, BitOrAssign, BitXorAssign, SubAssign};
-use std::{fmt, ops::Range};
+use std::fmt;
+use std::ops::{
+    BitAnd, BitAndAssign, BitOr, BitOrAssign, BitXor, BitXorAssign, RangeInclusive, Sub, SubAssign,
+};
 
 use super::store::{self, Store};
 use super::util;
@@ -20,11 +22,7 @@ pub struct Iter<'a> {
 
 impl Container {
     pub fn new(key: u16) -> Container {
-        Container {
-            key,
-            len: 0,
-            store: Store::Array(Vec::new()),
-        }
+        Container { key, len: 0, store: Store::Array(Vec::new()) }
     }
 }
 
@@ -39,13 +37,7 @@ impl Container {
         }
     }
 
-    pub fn insert_range(&mut self, range: Range<u16>) -> u64 {
-        // If the range is larger than the array limit, skip populating the
-        // array to then have to convert it to a bitmap anyway.
-        if matches!(self.store, Store::Array(_)) && range.end - range.start > ARRAY_LIMIT as u16 {
-            self.store = self.store.to_bitmap()
-        }
-
+    pub fn insert_range(&mut self, range: RangeInclusive<u16>) -> u64 {
         let inserted = self.store.insert_range(range);
         self.len += inserted;
         self.ensure_correct_store();
@@ -75,12 +67,8 @@ impl Container {
         }
     }
 
-    pub fn remove_range(&mut self, start: u32, end: u32) -> u64 {
-        debug_assert!(start <= end);
-        if start == end {
-            return 0;
-        }
-        let result = self.store.remove_range(start, end);
+    pub fn remove_range(&mut self, range: RangeInclusive<u16>) -> u64 {
+        let result = self.store.remove_range(range);
         self.len -= result;
         self.ensure_correct_store();
         result
@@ -118,6 +106,17 @@ impl Container {
     }
 }
 
+impl BitOr<&Container> for &Container {
+    type Output = Container;
+
+    fn bitor(self, rhs: &Container) -> Container {
+        let store = BitOr::bitor(&self.store, &rhs.store);
+        let mut container = Container { key: self.key, len: store.len(), store };
+        container.ensure_correct_store();
+        container
+    }
+}
+
 impl BitOrAssign<Container> for Container {
     fn bitor_assign(&mut self, rhs: Container) {
         BitOrAssign::bitor_assign(&mut self.store, rhs.store);
@@ -134,6 +133,17 @@ impl BitOrAssign<&Container> for Container {
     }
 }
 
+impl BitAnd<&Container> for &Container {
+    type Output = Container;
+
+    fn bitand(self, rhs: &Container) -> Container {
+        let store = BitAnd::bitand(&self.store, &rhs.store);
+        let mut container = Container { key: self.key, len: store.len(), store };
+        container.ensure_correct_store();
+        container
+    }
+}
+
 impl BitAndAssign<Container> for Container {
     fn bitand_assign(&mut self, rhs: Container) {
         BitAndAssign::bitand_assign(&mut self.store, rhs.store);
@@ -150,6 +160,17 @@ impl BitAndAssign<&Container> for Container {
     }
 }
 
+impl Sub<&Container> for &Container {
+    type Output = Container;
+
+    fn sub(self, rhs: &Container) -> Container {
+        let store = Sub::sub(&self.store, &rhs.store);
+        let mut container = Container { key: self.key, len: store.len(), store };
+        container.ensure_correct_store();
+        container
+    }
+}
+
 impl SubAssign<&Container> for Container {
     fn sub_assign(&mut self, rhs: &Container) {
         SubAssign::sub_assign(&mut self.store, &rhs.store);
@@ -158,6 +179,17 @@ impl SubAssign<&Container> for Container {
     }
 }
 
+impl BitXor<&Container> for &Container {
+    type Output = Container;
+
+    fn bitxor(self, rhs: &Container) -> Container {
+        let store = BitXor::bitxor(&self.store, &rhs.store);
+        let mut container = Container { key: self.key, len: store.len(), store };
+        container.ensure_correct_store();
+        container
+    }
+}
+
 impl BitXorAssign<Container> for Container {
     fn bitxor_assign(&mut self, rhs: Container) {
         BitXorAssign::bitxor_assign(&mut self.store, rhs.store);
@@ -179,10 +211,7 @@ impl<'a> IntoIterator for &'a Container {
     type IntoIter = Iter<'a>;
 
     fn into_iter(self) -> Iter<'a> {
-        Iter {
-            key: self.key,
-            inner: (&self.store).into_iter(),
-        }
+        Iter { key: self.key, inner: (&self.store).into_iter() }
     }
 }
 
@@ -191,10 +220,7 @@ impl IntoIterator for Container {
     type IntoIter = Iter<'static>;
 
     fn into_iter(self) -> Iter<'static> {
-        Iter {
-            key: self.key,
-            inner: self.store.into_iter(),
-        }
+        Iter { key: self.key, inner: self.store.into_iter() }
     }
 }
 
