@@ -11,6 +11,7 @@ mod iter;
 mod ops;
 mod serialization;
 
+use self::cmp::Pairs;
 pub use self::iter::IntoIter;
 pub use self::iter::Iter;
 
@@ -30,7 +31,7 @@ pub use self::iter::Iter;
 /// rb.insert(7);
 /// println!("total bits set to true: {}", rb.len());
 /// ```
-#[derive(PartialEq, Clone)]
+#[derive(PartialEq)]
 pub struct RoaringBitmap {
     containers: Vec<container::Container>,
 }
