@@ -1,10 +1,10 @@
+use std::collections::btree_map::{BTreeMap, Entry};
+use std::ops::RangeBounds;
+
 use crate::RoaringBitmap;
 use crate::RoaringTreemap;
 
 use super::util;
-use std::collections::btree_map::Entry;
-use std::collections::BTreeMap;
-use std::ops::Range;
 
 impl RoaringTreemap {
     /// Creates an empty `RoaringTreemap`.
@@ -16,9 +16,7 @@ impl RoaringTreemap {
     /// let mut rb = RoaringTreemap::new();
     /// ```
     pub fn new() -> RoaringTreemap {
-        RoaringTreemap {
-            map: BTreeMap::new(),
-        }
+        RoaringTreemap { map: BTreeMap::new() }
     }
 
     /// Adds a value to the set. Returns `true` if the value was not already present in the set.
@@ -35,10 +33,7 @@ impl RoaringTreemap {
     /// ```
     pub fn insert(&mut self, value: u64) -> bool {
         let (hi, lo) = util::split(value);
-        self.map
-            .entry(hi)
-            .or_insert_with(RoaringBitmap::new)
-            .insert(lo)
+        self.map.entry(hi).or_insert_with(RoaringBitmap::new).insert(lo)
     }
 
     /// Pushes `value` in the treemap only if it is greater than the current maximum value.
@@ -60,10 +55,7 @@ impl RoaringTreemap {
     /// ```
     pub fn push(&mut self, value: u64) -> bool {
         let (hi, lo) = util::split(value);
-        self.map
-            .entry(hi)
-            .or_insert_with(RoaringBitmap::new)
-            .push(lo)
+        self.map.entry(hi).or_insert_with(RoaringBitmap::new).push(lo)
     }
 
     /// Removes a value from the set. Returns `true` if the value was present in the set.
@@ -96,12 +88,9 @@ impl RoaringTreemap {
         }
     }
 
-    /// Removes a range of values from the set specific as [start..end).
+    /// Removes a range of values.
     /// Returns the number of removed values.
     ///
-    /// Note that due to the exclusive end you can't remove the item at the
-    /// last index (u64::MAX) using this function!
-    ///
     /// # Examples
     ///
     /// ```rust
@@ -112,35 +101,28 @@ impl RoaringTreemap {
     /// rb.insert(3);
     /// assert_eq!(rb.remove_range(2..4), 2);
     /// ```
-    pub fn remove_range(&mut self, range: Range<u64>) -> u64 {
-        if range.start == range.end {
-            return 0;
-        }
+    pub fn remove_range<R>(&mut self, range: R) -> u64
+    where
+        R: RangeBounds<u64>,
+    {
+        let (start, end) = match util::convert_range_to_inclusive(range) {
+            Some(range) => (*range.start(), *range.end()),
+            None => return 0,
+        };
+
+        let (start_container_key, start_index) = util::split(start);
+        let (end_container_key, end_index) = util::split(end);
+
         let mut keys_to_remove = Vec::new();
         let mut removed = 0;
-        // inclusive bounds for start and end
-        let (start_hi, start_lo) = util::split(range.start);
-        let (end_hi, end_lo) = util::split(range.end - 1);
+
         for (&key, rb) in &mut self.map {
-            if key >= start_hi && key <= end_hi {
-                let a = if key == start_hi {
-                    u64::from(start_lo)
-                } else {
-                    0
-                };
-                let b = if key == end_hi {
-                    u64::from(end_lo) + 1 // make it exclusive
-                } else {
-                    u64::from(u32::max_value()) + 1
-                };
-                if a == 0 && b == u64::from(u32::max_value()) + 1 {
-                    removed += rb.len();
+            if key >= start_container_key && key <= end_container_key {
+                let a = if key == start_container_key { start_index } else { 0 };
+                let b = if key == end_container_key { end_index } else { u32::MAX };
+                removed += rb.remove_range(a..=b);
+                if rb.is_empty() {
                     keys_to_remove.push(key);
-                } else {
-                    removed += rb.remove_range(a..b);
-                    if rb.is_empty() {
-                        keys_to_remove.push(key);
-                    }
                 }
             }
         }
@@ -277,3 +259,13 @@ impl Default for RoaringTreemap {
         RoaringTreemap::new()
     }
 }
+
+impl Clone for RoaringTreemap {
+    fn clone(&self) -> Self {
+        RoaringTreemap { map: self.map.clone() }
+    }
+
+    fn clone_from(&mut self, other: &Self) {
+        self.map.clone_from(&other.map);
+    }
+}
